/*
 * LastChangesTaglet
 *
 * Copyright (c) 2005 Eugene Matyushkin (E-mail: skipy@mail.ru)
 *
 * License agreement:
 *
 * 1. This code is published AS IS. Author is not responsible for any damage that can be
 *    caused by any application that uses this code.
 * 2. Author does not give a garantee, that this code is error free.
 * 3. This code can be used in NON-COMMERCIAL applications AS IS without any special
 *    permission from author.
 * 4. This code can be modified without any special permission from author IF AND ONLY IF
 *    this license agreement will remain unchanged.
 * 5. SPECIAL PERMISSION for this code usage in COMMERCIAL application SHOULD be obtained
 *    from author.
 */
package ru.skipy.taglet;

import com.sun.tools.doclets.Taglet;    // Taglet API
import com.sun.javadoc.*;               // Doclet API
import java.util.Map;

/**
 * This class implements {@link Taglet} interface to process newly introduced
 * javadoc tag <i>&#64;lastChanges</i>. It's just an example of how new tags can
 * be used to reflect custom information in the documentation. Go to
 * {@link #toString(com.sun.javadoc.Tag)} and {@link #toString(com.sun.javadoc.Tag[])}
 * methods docs to see how it looks like.
 *
 * @author  Eugene Matyushkin
 *
 * @version 1.0
 */
public class LastChangesTaglet implements Taglet{

    /**
     * Tag name
     */
    private static final String NAME = "lastChanges";

    /**
     * Tag header in documentation
     */
    private static final String HEADER = "Last changes:";
    

    /**
     * @return always <code>true</code>, since this tag can be used in constructor javadoc
     * {@inheritDoc}
     */
    public boolean inConstructor() {
        return true;
    }


    /**
     * @return always <code>true</code>, since this tag can be used in field javadoc
     * {@inheritDoc}
     */
    public boolean inField() {
        return true;
    }


    /**
     * @return always <code>true</code>, since this tag can be used in method javadoc
     * {@inheritDoc}
     */
    public boolean inMethod() {
        return true;
    }


    /**
     * @return always <code>true</code>, since this tag can be used in overview
     * {@inheritDoc}
     */
    public boolean inOverview() {
        return true;
    }


    /**
     * @return always <code>true</code>, since this tag can be used in package javadoc
     * {@inheritDoc}
     */
    public boolean inPackage() {
        return true;
    }


    /**
     * @return always <code>true</code>, since this tag can be used in class or interface
     * javadoc
     * {@inheritDoc}
     */
    public boolean inType() {
        return true;
    }


    /**
     * @return always <code>false</code>, since this tag s not inline
     * {@inheritDoc}
     */
    public boolean isInlineTag() {
        return false;
    }


    /**
     * @return name of the tag
     *
     * {@inheritDoc}
     */
    public String getName() {
        return NAME;
    }

    /**
     * @lastChanges (2004Jul01) Eugene Matyushkin
     * @param tag tag
     * @return string representation for single tag
     */

    public String toString( Tag tag ) {
        return "<DT><B><font color=\"blue\">" + HEADER + "</font></B><DD>"
               + "<table cellpadding=2 cellspacing=0><tr><td>"
               + "<code>" + tag.text() + "</code>"
               + "</td></tr></table></DD>\n";
    }

    /**
     * @lastChanges (2004Jun30) Eugene Matyushkin: table format changed
     * @lastChanges (2004Jul01) Eugene Matyushkin: bug fixed
     *
     * @param tags tags array
     * @return string representation for multiple tags
     */
    public String toString( Tag[] tags ) {
        if (tags.length == 0) {
            return null;
        }
        String result = "\n<DT><B><font color=\"blue\">" + HEADER + "</font></B><DD>";
        result += "<table cellpadding=2 cellspacing=0><tr><td>";
        for (int i = 0; i < tags.length; i++) {
            if (i > 0) {
                result += "</td></tr><tr><td>";
            }
            result += "<code>"+tags[i].text()+"</code>";
        }
        return result + "</td></tr></table></DD>\n";
    }

    /**
     * This method is used to register tag in the taglet map
     *
     * @param tagletMap map, in which this tag should be placed (with tag name as key)
     */
    public static void register(Map tagletMap) {
       LastChangesTaglet tag = new LastChangesTaglet();
       Taglet t = (Taglet) tagletMap.get(tag.getName());
       if (t != null) {
           tagletMap.remove(tag.getName());
       }
       tagletMap.put(tag.getName(), tag);
    }

}
